from ..base import PowerSwitchDevice
import os
from subprocess import Popen, PIPE

class ClewareSwitch(PowerSwitchDevice):
    def __init__(self, usbswitchcmd_exe_path: str):
        super().__init__()
        self.exe = usbswitchcmd_exe_path

    def run_cmd(self, cmd) -> (int, str, str):
        args = [self.exe, cmd]
        p = Popen(args, stdout=PIPE, stderr=PIPE)
        p.wait()
        return p.returncode, p.stdout.read().decode("utf-8").strip("\n"), p.stderr.read().decode("utf-8").strip("\n")

    def status(self) -> bool:
        cmd = "-r"
        ret, out, err = self.run_cmd(cmd)
        print(f"status: ret: {ret}, out: '{out}', err: '{err}'")
        if out == "1":
            return True
        elif out == "0":
            return False
        else:
            raise RuntimeError(f"Failed to read USB switch status.\nCommand: '{cmd}'\nCode: {ret}, stdout: '{out}', stderr: '{err}'")

    def on(self):
        cmd = "1"
        ret, out, err = self.run_cmd(cmd)
        if ret != 1 or out or err:  # yes, it actually returns 1 when it turns on ._.
            raise RuntimeError(f"Failed to turn on USB switch.\nCommand: '{cmd}'\nCode: {ret}, stdout: '{out}', stderr: '{err}'")

    def off(self):
        cmd = "0"
        ret, out, err = self.run_cmd(cmd)
        if ret != 0 or out or err:
            raise RuntimeError(f"Failed to turn on USB switch.\nCommand: '{cmd}'\nCode: {ret}, stdout: '{out}', stderr: '{err}'")

    def __str__(self):
        return f"Cleware USB Switch"